import './Checkers.css'
import React from 'react'

export const Color = {
    white: "WHITE",
    black: "BLACK",

    opposite: (color) => {
        if (color === Color.white)
            return Color.black;
        if (color === Color.black)
            return Color.white;

        return color;
    }
};

/*
 * Stone
 */
export function Stone({ color, type }) {
    switch (color) {
        case Color.white:
            return <WhiteStone type={type} />;

        case Color.black:
            return <BlackStone type={type} />;

        case '':
        case undefined:
        case null:
            return; // no stone :)

        default:
            console.warn("Unknown color: ", color)
    }
}

export function WhiteStone({ type }) {
    if (type === 'KING')
        return <span className="Stone white king">⛁</span>
    else
        return <span className="Stone white man">⛀</span>
}

export function BlackStone({ type }) {
    if (type === 'KING')
        return <span className="Stone white king">⛃</span>
    else
        return <span className="Stone black man">⛂</span>
}

/*
 * Player
 */
export function Player({ color, name }) {
    return (
        <div className='Player'>
            {name}
            <Stone color={color} />
        </div>
    )
}

/*
 * Board
 */
export function Board({ game, onClick }) {
    const board = (game && game.board && typeof game.board === 'object') ? game.board : defaultBoard;

    const BlackTile = () => {
        return <div className='Tile black' />
    }

    const WhiteTile = ({ id }) => {
        const stone = board[id];
        const isInteractive = (typeof onClick === 'function') ? ' interactive' : '';

        return (
            <div className={'Tile white' + isInteractive} onClick={() => isInteractive && onClick(id)}>
                <Stone color={stone?.color} type={stone?.type} />
            </div>
        );
    }

    return <div className={'Board' + (game?.myColor === Color.black ? ' flip' : '')} >
        <div className='row'>
            <BlackTile /> <WhiteTile id={1} />
            <BlackTile /> <WhiteTile id={2} />
            <BlackTile /> <WhiteTile id={3} />
            <BlackTile /> <WhiteTile id={4} />
        </div>
        <div className='row'>
            <WhiteTile id={5} /> <BlackTile />
            <WhiteTile id={6} /> <BlackTile />
            <WhiteTile id={7} /> <BlackTile />
            <WhiteTile id={8} /> <BlackTile />
        </div>
        <div className='row'>
            <BlackTile /> <WhiteTile id={9} />
            <BlackTile /> <WhiteTile id={10} />
            <BlackTile /> <WhiteTile id={11} />
            <BlackTile /> <WhiteTile id={12} />
        </div>
        <div className='row'>
            <WhiteTile id={13} /> <BlackTile />
            <WhiteTile id={14} /> <BlackTile />
            <WhiteTile id={15} /> <BlackTile />
            <WhiteTile id={16} /> <BlackTile />
        </div>

        <div className='row'>
            <BlackTile /> <WhiteTile id={17} />
            <BlackTile /> <WhiteTile id={18} />
            <BlackTile /> <WhiteTile id={19} />
            <BlackTile /> <WhiteTile id={20} />
        </div>
        <div className='row'>
            <WhiteTile id={21} /> <BlackTile />
            <WhiteTile id={22} /> <BlackTile />
            <WhiteTile id={23} /> <BlackTile />
            <WhiteTile id={24} /> <BlackTile />
        </div>
        <div className='row'>
            <BlackTile /> <WhiteTile id={25} />
            <BlackTile /> <WhiteTile id={26} />
            <BlackTile /> <WhiteTile id={27} />
            <BlackTile /> <WhiteTile id={28} />
        </div>
        <div className='row'>
            <WhiteTile id={29} /> <BlackTile />
            <WhiteTile id={30} /> <BlackTile />
            <WhiteTile id={31} /> <BlackTile />
            <WhiteTile id={32} /> <BlackTile />
        </div>
    </div>
}

const blackMan = { color: Color.black, type: 'MAN' };
const blackKing = { color: Color.black, type: 'KING' };
const whiteMan = { color: Color.white, type: 'MAN' };
const whiteKing = { color: Color.white, type: 'KING' };

export const defaultBoard = {
    1: blackMan, 2: blackMan, 3: blackMan, 4: blackMan, 5: blackMan, 6: blackMan, 7: blackMan, 8: blackMan, 9: blackMan, 10: blackMan, 11: blackMan, 12: blackMan,
    21: whiteMan, 22: whiteMan, 23: whiteMan, 24: whiteMan, 25: whiteMan, 26: whiteMan, 27: whiteMan, 28: whiteMan, 29: whiteMan, 30: whiteMan, 31: whiteMan, 32: whiteMan,
}

export function nextStone(currentStone) {
    const color = currentStone?.color;
    const type = currentStone?.type;

    if (!color || !type)
        return blackMan;

    if (color === Color.black && type === 'MAN')
        return blackKing;

    if (color === Color.black && type === 'KING')
        return whiteMan;

    if (color === Color.white && type === 'MAN')
        return whiteKing;

    if (color === Color.white && type === 'KING')
        return null;
}