import React, { useContext, useState } from 'react';
import { useLocation, matchPath } from 'react-router-dom';
import { GamesContext } from '../../context/games';

import { Color, Player, Board, nextStone } from '../../components/Checkers';
import './GameBoard.css';

export default function GameBoard({ username, onNewGameBoardStone, onActiveGameMove }) {
  const games = useContext(GamesContext);
  const { pathname } = useLocation();
  const [startId, setStartId] = useState(null);

  const onClick_NewGame = (cellId) => {
    let nextBoard = { ...games.newGame.board };
    nextBoard[cellId] = nextStone(nextBoard[cellId]);

    onNewGameBoardStone(nextBoard);
  }

  const onClick_ActiveGame = (cellId) => {
    if (startId === cellId)
      return setStartId(null);

    if (startId === null)
      return setStartId(cellId);

    if (game?.uuid)
      onActiveGameMove(game.uuid, startId, cellId);

    setStartId(null);
  };

  const [game, onClick] = (() => {
    if (matchPath('/games/new', pathname))
      return [games.newGame, onClick_NewGame];

    if (matchPath('/games/proposal', pathname))
      return [games.findGame({ uuid: games.proposal.selectedUUID }), null];
    else if (matchPath('/games/active', pathname))
      return [games.findGame({ uuid: games.active.selectedUUID }), onClick_ActiveGame];
    else if (matchPath('/games/archive', pathname))
      return [games.findGame({ uuid: games.archive.selectedUUID }), null];

    return [{}, null];
  })(); // <<-- Execute

  const opponentColor = Color.opposite(game?.myColor);
  const [opponentName, myName] = game?.opponentName ? [game.opponentName, username] : ['', ''];

  const optionalOnClick = (onClick && game?.board) ? (id) => onClick(id) : null;

  return (
    <div className='GameBoard'>
      <Player color={opponentColor || Color.black} name={opponentName} />
      <Board game={game} onClick={optionalOnClick} />
      <Player color={game?.myColor || Color.white} name={myName} />
    </div>
  )
}